'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _RoutineStatus = require('./RoutineStatus');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Class for running routines at set intervals
 * @param routine - The routine to run
 * @param count - # of times to run routine
 * @param interval - how often to run routine in milliseconds.
 */

var RoutineRunner = function () {
    function RoutineRunner(routine, count, interval) {
        _classCallCheck(this, RoutineRunner);

        this._routine = routine;
        this._count = count;
        this._interval = interval;

        this._intervalTimer = null;
        this._runningRoutines = {};
        this._currentRoutineId = 1;
    }

    _createClass(RoutineRunner, [{
        key: 'start',
        value: function start() {
            var _this = this;

            this.startRoutine(this._currentRoutineId);
            return new Promise(function (resolve) {
                _this._intervalTimer = setInterval(function () {
                    _this._currentRoutineId = _this._currentRoutineId + 1;

                    if (_this._currentRoutineId >= _this._count) {
                        _this.stop();
                        resolve();
                    }

                    _this.startRoutine(_this._currentRoutineId);
                }, _this._interval);
            });
        }
    }, {
        key: 'stop',
        value: function stop() {
            clearInterval(this._intervalTimer);
        }
    }, {
        key: 'startRoutine',
        value: function startRoutine(id) {
            var _this2 = this;

            if (!this._runningRoutines.hasOwnProperty(id)) {
                this.updateRoutine(id, _RoutineStatus.RoutineStatus.RUNNING);
                this._routine.execute().then(function (r) {
                    _this2.updateRoutine(id, _RoutineStatus.RoutineStatus.COMPLETED);
                });
            }
        }
    }, {
        key: 'updateRoutine',
        value: function updateRoutine(id, action) {
            if (action === _RoutineStatus.RoutineStatus.COMPLETED && !this._runningRoutines.hasOwnProperty(id)) {
                return false;
            }

            switch (action) {
                case _RoutineStatus.RoutineStatus.RUNNING:
                    this._runningRoutines[id] = {
                        status: _RoutineStatus.RoutineStatus.RUNNING,
                        started: Date()
                    };
                    break;
                case _RoutineStatus.RoutineStatus.COMPLETED:
                    this._runningRoutines[id] = _extends({}, this._runningRoutines[id], {
                        status: _RoutineStatus.RoutineStatus.COMPLETED,
                        completed: Date()
                    });
                    break;
                default:
                    break;
            }

            return true;
        }
    }]);

    return RoutineRunner;
}();

exports.default = RoutineRunner;